; 11/20/93
;
; About CHKDRV.COM (it can be renamed !)
;
; It appears that the program (CHKDRV.COM) can't address the hardware if
; it uses PD's Dos environment so it needs to load its own. This means
; that NoShell option must not be used, but this prevents Paradox from
; receiving the errorlevel from the OS.
; In order to get the floppy drive(s) status and whether a diskette placed
; in a drive has a write protected tab, the PAL program must Play a .SC
; file that CHKDRV creates in current directory. The file is RREETT.SC
; (hopping there isn't a file with the same name... I hard coded this
; stupid name...)
;
; CHKDRV command line switches and arguments specified are case insensitive.
; No need for space between switches although they make the call clearer..
;
; Switches and usage:
; /h or -h  Displays switch usage. DON'T use it from Paradox, doesn't do
;           anything... if NoRefresh is used
; /d or -d  Specifies drive to be checked. If this switched is not used
;           the program uses default A. Arguments possible are A or B
;           Usage: CHKDRV /db   check drive B
; /t or -t  What to check. Arguments are R or W (check if Ready, check if
;           Write on diskette is possible) R is default, of course an "R"
;           check is always performed...
;           Usage: CHKDRV /tw   check if disk in drive A is write protected
; /v or -v  Specify that the values outputed in .SC be assigned to variables.
;           Default is in a dynarray. Switch /v might help those who use
;           Paradox < 4.0.
;
;           Here is the list of the variables or dynarray tags found in .SC
;           file (depending whether switch /v was used):
;
;           without /v                        with /v
;           ------------------------------------------------------
;           Ready.n                           y["Ready.n"]
;           Room.n                            y["Room.n"]
;           Bytes.n                           y["Bytes.n"]
;           Sectors.n                         y["Sectors.n"]
;           TotalClusters.n                   y["TotalClusters.n"]
;           FreeClusters.n                    y["FreeClusters.n"]
;
;           All values returned in vars. or DY are numbers.
;           Values in Ready.n var. or dynarray element can be:
;                  0 for Ready. It also means Write OK if switch "/t" with
;                  argument "w" was used.
;                  1 for drive Not ready
;                  2 for diskette Write protected
;
; /r or -r  This will write a line in .SC file: room.n = n where "n" is
;           the space available on the floppy. This is the space that dos
;           reports when dir command is used.

; /i or -i  This will write disk information (on A or specifyied in /d)
;           in .SC file. Returned information is:
;              Number of bytes per sector (this is always 512 in DOS)
;              Number of sectors per cluster.
;              Number of total clusters per disk
;              Number of free clusters on disk
;           (see variable names and dynarray tags above)
;           With a little math then it is easy to find the type of disk
;           drive, true amount needed for files on floppy when their size
;           was read with FileSize() or in 4.5 FamilySize() which both
;           report clusters.
;           Also the type of drive... a 360Kb (who uses it ?) always stores
;           two sectors within each cluster, while a 1.2Mb only one...
;
; Since this utility is a quick thing it doesn't do any fancy parsing on
; switches so keep them at one letter.
;
; It takes either 1, 2, 3, 4, 5 or no switches, depending of needs... The
; order in which switches are fed or spacing doesn't matter. (leave a space
; between the program name and switches)
;
; CHKDRV can be in current directory or can be called with a full path. For
; the latter use duble slashes: Run NoRefresh "e:\\util\\pdutl\\chkdrv /tw"
;
; Call Examples (assumes current directory)
; * Check if drive A is ready:
;            Run NoRefresh "chkdrv"
; * Check if diskette in drive A can be written:
;            Run NoRefresh "chkdrv /tw"
; * Check if diskette in drive B can be written:
;            Run NoRefresh "chkdrv /db /tw"
; * Check if drive A is ready and use variable names in output .sc file
;            Run NoRefresh "chkdrv /v"
; * Get drive info for E drive
;            Run NoRefresh "chkdrv /de /r /i"
;            Here is the content of the .SC file taken from my E drive test:
;            Ready.n=0
;            Room.n= 95465472
;            Bytes.n= 512
;            Sectors.n= 8
;            TotalClusters.n= 51141
;            FreeClusters.n= 23307
;            If we do the math: Bytes.n x Sectors.n x FreeClusters.n the
;            result equals Room.n So if more info is needed than switch /r
;            can be left out and math on variables (using /i) would give
;            everything...
;
;      NOTE: If you have a write protected diskette in a drive and call this
;            program without /t switch the variable returned will be 0 (zero)
;            so IF WRITE PROTECT CHECK IS NEEDED USE /tw
;
; Also... because shelling out and doing this type of things from Paradox is
; not on the fast side... perhaps augmenting the routine which uses this
; utility with some messages would divert user's attention from the speed...
;
; Looking at the second example it appears that CHKDRV checks if the drive
; is ready firts and in fact this is what it always does.. The reason for /t
; switch is that if we don't need to write to the disk we shouldn't use
; /t switch which adds more work...
;
; To test this in Dos, run the program with adequate switches then look at
; the RREETT.SC file created in current directory (type rreett.sc).
;
; As a general rule it is always better to do 'things' the Paradox way, so if
; there's a function or command available in PD that should be used. PD 4.5
; fixes flaws that we found in previous versions of some i/o related functions
; (such as DirExist). Also, as David Berg of Borland suggested in a discussion
; on Pdoxdos forum (CompuServe) such utilities which use dos/bios calls might
; not be reliable on net drives, cd-roms or might trigger anti-virus software,
; if installed. The main purpose of this .com is to check the floppy drives
; A and B for readiness and write protection of disketts when a pal program
; writes to the floppy. However, this program does work on local hard drives
; and their logical partitions (besides floppy drives) and I also got readings
; on my cd-rom: using CHKDRV /di /i ("i" in /d switch the cd drive letter
; assignment) it reported 2kb per sector, 8 sectors per cluster, the total
; number of clusters and 0 free clusters !
;
; This is freeware, use it at your own risk, etc. !
; Michael Radulescu CIS 71561,3647


   ;-- A simple script to test CHKDRV

   ;--------------------------------------------------------------------
   ; If you make the call: "chkdrv" and there's a diskette with a write
   ; protected tab in drive A and the drive is cloased then the variable
   ; Ready.n (or y["Ready.n"] in .SC file will be 0 which means OK. But
   ; if your intention is to write to the floppy Paradox will bomb..
   ; so use "/t" switch !!!!!!!!
   ;--------------------------------------------------------------------

   ;-- Check diskette in A: if write protected, return info
   Run NoRefresh "chkdrv /tw /i"
   Play "RREETT"               ; Play script to get values

   ;-- Use this below for PD > 3.5. Assuming switch /v not used !!!
   ;   Perhaps for a single variable such as Ready.n it might
   ;   be better to use switch /v so we don't load a DY for one var.

   n = DynarraySize(y)         ; "y" is the dynarray, if any
                               ;  call above will return a DY
   n = Iif(n>2,10,DynarraySize(y)+3)
   n1 = 0
   Dynarray a[]
   a["Title"] = "Drive - Diskette Status"
   Window Create @5,5 Height n Width 40 To w
   Window SetAttributes w From a
   Foreach tag.a in y
     n1 = n1 + 1
     txt.a = Iif(tag.a="READY.N",
             Iif(y[tag.a]<2,Iif(y[tag.a]<1,"Drive Ready","Drive Not ready"),
             "Write Protected"),"")
     @n1,2 ?? tag.a+" = "+Strval(y[tag.a])+"   "+txt.a
   Endforeach
   ;-- We can zap the file here although it
   ;   is ovewritten at each call to CHKDRV.
   Editor New "RREETT.SC"
   {Cancel} {Yes}
   Echo Normal Echo Off
   Message "Hit a key..."
   x = GetChar()
   Window Close
   Release Vars a,n,n1,y,tag.a,txt.a,x
   ;------------------------------------------------------------------------

   ;-- Un-remark below, comment above for PD 3.5
;   ;-- Check diskette in A: if write protected, return info
;   Run NoRefresh "chkdrv /tw /i /v" ; Must use /v for PD < 4.0

;   Play "RREETT"                    ; Play script to get values
;   n = 1                            ; At least one variable
;   Switch
;     Case IsAssigned(Room.n):
;        n = n + 1
;     Case IsAssigned(Bytes.n):
;        n = n + 4     ; There are 4 vars for drive info
;   Endswitch
;   Echo Normal
;   PaintCanvas Attribute 30 5,5,5+n+2,45
;   Style Attribute 30
;   If Ready.n < 2 Then
;      txt.a = Substr("Drive Ready",Search("-",Ready.n-1),25)+
;              Substr("Drive Not Ready",1-Search("-",Ready.n-1),25)
;   Else
;      txt.a = "Write Protected"
;   Endif
;   row.n = 6
;   @row.n,8 ?? "Ready.n = "+StrVal(Ready.n)+"  "+txt.a
;   If IsAssigned(Room.n) Then
;      row.n = row.n + 1
;      @row.n,8 ?? "Room.n = "+StrVal(Room.n)
;   Endif
;   If IsAssigned(Bytes.n) Then
;      ;-- List'em
;      row.n = row.n + 1
;      @row.n,8 ?? "Bytes.n = "+StrVal(Bytes.n)
;      row.n = row.n + 1
;      @row.n,8 ?? "Sectors.n = "+StrVal(Sectors.n)
;      row.n = row.n + 1
;      @row.n,8 ?? "TotalClusters.n = "+StrVal(TotalClusters.n)
;      row.n = row.n + 1
;      @row.n,8 ?? "FreeClusters.n = "+StrVal(FreeClusters.n)
;   Endif
;   Style
;   ;-- We can zap the file here although it
;   ;   is ovewritten at each call to CHKDRV.
;   Run NoRefresh "Del RREETT.SC"
;   Message "Hit a key..."
;   x = GetChar()
;   Release Vars n,txt.a,row.n,x,
;                Ready.n,Room.n,
;                Bytes.n,Sectors.n,
;                TotalClusters.n,FreeClusters.n
